; $Id: davedat__define.pro,v 1.3.4.1 2004/05/27 22:50:54 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   DaveDat::cleanup
;
; PURPOSE:____________________
;   Cleanup method for the DaveDat class
;
; INPUTS:_____________________
;   None.
;
pro DaveDat::cleanup

; use heap_free recursively deletes a pointer
if (ptr_valid(self.davePtr)) then heap_free, self.davePtr 
if (ptr_valid(self.descriPtr)) then ptr_free, self.descriPtr
if (ptr_valid(self.treatPtr)) then ptr_free, self.treatPtr
if (ptr_valid(self.xvPtr)) then ptr_free, self.xvPtr
if (ptr_valid(self.zvPtr)) then ptr_free, self.zvPtr
if (ptr_valid(self.zePtr)) then ptr_free, self.zePtr

end


;===============================================================================
; NAME:_______________________
;   DaveDat::init
; 
; PURPOSE:
;   Initialization method for the DaveDat class
;
; INPUTS:_____________________
;   KEYWORDS:
;
;     filename: The name of the file from which the (DAVE format) data
;     should be retrieved and used for the construction of the object
;
;     nameTag: A string name tag for object
;
;     davePtr: Specifies a ptr to a (DAVE) structure to be used in
;     constructing the current object
;
;     scalefactor: a multiplicative scalefactor to be assigned to the
;     object data.
;
; OUTPUTS:____________________
;
;     return value: 0 signifies failure and 1 success.
;
function DaveDat::init, filename=fname, davePtr=dPtr, scalefactor=sfactor, nameTag=nameTag

; reset !error_state structure, ie clear all errors
message,/reset

;; caller should not specify both filename and davePtr as this is ambiguous
if ((n_elements(fname)  gt 0 ) and (n_elements(dPtr) gt 0)) then begin
    !error_state.msg = 'DaveDat::init: Ambiguous call - cannot specify both filename and davePtr!'
    !error_state.code = -1
    return, 0
endif

if (n_elements(fname) gt 0) then begin
    ;; make sure fname exists
    ffname = findfile(fname,count=cnt)
    if (cnt ge 1) then begin
        self.filename = ffname
        self.nameTag = ffname
        restore,ffname
        ;; make sure a valid davePtr was restored
        if (ptr_valid(davePtr)) then begin
            self.davePtr = davePtr
        endif else begin
            !error_state.msg = 'DaveDat::init: "'+fname+'" does not contain a valid davePtr'
            !error_state.code = -1
            return, 0
        endelse
    endif else begin
        !error_state.msg = 'DaveDat::init: file "'+fname+'" not found'
        !error_state.code = -1
        return, 0
    endelse

    ;; should have a davePtr by now so initialize rest of class variables
    if (self->DaveDat::resetData() eq 0) then return, 0
endif

if (n_elements(nameTag)) then begin
    if (vartype(nameTag) eq 'STRING') then self.nameTag = nameTag
endif

if (n_elements(dPtr) gt 0) then begin
    if (ptr_valid(dPtr)) then begin
        dave_beast_ops,'dup',dPtr,dPtr_copy ; make a copy of dPtr
        self.davePtr = dPtr_copy ; NOTE: a _copy_ of dPtr is put in object, not dPtr itself
    endif else begin
        !error_state.msg = 'DaveDat::init: invalid pointer (davePtr) passed'
        !error_state.code = -1
        return, 0
    endelse
    ;; should have a davePtr by now so initialize rest of class variables
    if (self->DaveDat::resetData() eq 0) then return, 0
endif

if (n_elements(sfactor) gt 0) then begin
    self.scaleFactor = sfactor
endif else begin
    self.scaleFactor = 1.0
endelse

return, 1

end


;===============================================================================
; NAME:_______________________
;   DaveDat__define
;
; PURPOSE:____________________
;   Definition of the class structure for the DaveDat class
;
; INPUTS:_____________________
;   None.
;
pro DaveDat__define

void = {DaveDat, $
        nameTag:'', $           ; a string name tag for object
        nReferences:0b, $       ; nos references to this object
        davePtr: ptr_new(), $   ; davePtr as defined within DAVE
        descriPtr: ptr_new(), $ ; ptr to 'descriPtr' structure
        filename:'', $          ; if applicable, filename from which data obtained
        scaleFactor: 1.0, $     ; scalefactor for this data
        instrument:'',$         ; string containing instrument name
        treatPtr:ptr_new(), $   ; string arr containing data treatment record 
        xvPtr:ptr_new(), $      ; primary independent axis values (x-axis)
        xLabel:'', $            ; x-axis label eg 'Energy transfer'
        xType:'', $             ; x-axis type 'points' or 'hist'
        xUnits:'', $            ; x-axis units eg 'meV'
        xDir:0b, $              ; 0=original x was increasing; 1=x was decreasing with index
        zvPtr:ptr_new(), $      ; dependent values (z-axis)
        zePtr:ptr_new(), $      ; z-axis errors
        zLabel:'', $            ; z-axis labels
        zUnits:'' $             ; z-axis units
       }
end

