; $Id: slab_ss_calc.pro,v 1.3 2004/05/03 15:38:38 john Exp $
;************************************************************************************************
; Program to calculate SAFs and SSFs for a plane slab sample in a "thin" container
; by John Copley
;
;	slab_ss_calc,sigmat,psi0,tth,ssf=ssf,saf=saf
; sigmat:				the dimensionless product sigmaT*t
;	psi0, psi:	are angles in degrees
; tth:			  is the array that contains the diffraction angles where SSF and SAF are to be calculated
;     								(note that tth is a double precission array)
;
; ! Default values are created if parameters were not defined on input
;************************************************************************************************
pro slab_ss_calc,sigmat,psi0,tth,ssf=ssf,saf=saf
;************************************************************************************************
;
compile_opt strictarr
;
;
; sigmat is the dimensionless product sigmaT*t
;	psi0 and psi are angles in degrees
;
; Create default values if parameters were not defined on input
if (n_elements(sigmat) eq 0) then sigmat=0.05
if (n_elements(psi0) eq 0) then psi0=60.0
if (n_elements(tth) eq 0) then begin
	start=28
	stop=32
	step=0.005
	n=(stop-start)/step+1
	tth=(start+findgen(n)*step)
endif
;
sigmat=double(sigmat)
psi0=double(psi0)
;
ntth=n_elements(tth)
saf=dblarr(ntth)
ssf=dblarr(ntth)
;
; If sigmat < 0 set saf and ssf = -1 and return.
if (sigmat lt 0.0) then begin
	saf=-1.0
	ssf=-1.0
	return
endif
;
; Test for sigmat = 0.
if (sigmat eq 0.0) then begin
	saf=1.0
	ssf=1.0
	return
endif
;
; Require that -!pi/2 < psi0 (in radians) <= !pi/2
psi0_in=psi0*!dtor
while (psi0_in gt !pi/2) do psi0_in=psi0_in-!pi
while (psi0_in le -!pi/2) do psi0_in=psi0_in+!pi
;
cos_psi0=cos(psi0_in)
if (cos_psi0 eq 0.0) then return
sec_psi0=1.0/cos_psi0
exp_psi0=exp(-sigmat*sec_psi0)
;
; Construct psi (in radians).
psi=psi0_in+tth*!dtor
;
; For all values of psi require that -!pi < psi <= !pi
repeat begin
	ind=where(psi gt !pi, count)
	if (count gt 0) then psi[ind]=psi[ind]-2*!pi
endrep until (count eq 0)
;
repeat begin
	ind=where(psi le -!pi, count)
	if (count gt 0) then psi[ind]=psi[ind]+2*!pi
endrep until (count eq 0)
;
cos_psi=cos(psi)
;
; Treat cos(psi) < 0 (reflection)
ind=where(cos_psi lt 0.0,count)
;print,'Reflection count: ',count
if (count gt 0) then begin
	sec_psi=1.0/cos_psi[ind]
	exp_psi=exp(sigmat*sec_psi)
	saf[ind]=0.5*(1.0+exp_psi0*exp_psi)
	ssf[ind]=(1.0-exp_psi0*exp_psi)/(sigmat*(sec_psi0-sec_psi))
endif
;
; Treat cos(psi) > 0 (transmission), with cos(psi) <> cos(psi0)
ind=where(cos_psi gt 0.0 and abs(cos_psi-cos_psi0) gt 0.00001,count)
;print,'Transmission count: ',count
if (count gt 0) then begin
	sec_psi=1.0/cos_psi[ind]
	exp_psi=exp(-sigmat*sec_psi)
	saf[ind]=0.5*(exp_psi0+exp_psi)
	ssf[ind]=(exp_psi-exp_psi0)/(sigmat*(sec_psi0-sec_psi))
endif
;
; Treat cos(psi) > 0 (transmission), with cos(psi) = cos(psi0)
ind=where(cos_psi gt 0.0 and abs(cos_psi-cos_psi0) le 0.00001,count)
;print,'Transmission parallel count: ',count
if (count gt 0) then begin
;	print,psi[ind]/!dtor
	saf[ind]=exp_psi0
	ssf[ind]=exp_psi0
endif
;
ind=sort(psi)
;window,0
;plot,psi[ind]/!dtor,saf[ind],psym=-4
;oplot,psi[ind]/!dtor,ssf[ind],psym=-5
;wshow,0

;print,psi[ind]/!dtor,saf[ind],ssf[ind]
end